{ system ? builtins.currentSystem
, nix-thunk ? import ./dep/nix-thunk {}
, obelisk ? import ./.obelisk/impl {
    inherit system;
    iosSdkVersion = "13.2";

    # Here we insert our own nixpkgs overlays. Since it's a bit awkward, a
    # future version of Obelisk will make this more straightforward (link?),
    # but for now, the way to do this is by modifying the reflex-platform
    # function to modify its nixpkgsOverlays argument.
    reflex-platform-func = (args: import (nix-thunk.thunkSource ./.obelisk/impl + "/dep/reflex-platform")
      (args // { nixpkgsOverlays = args.nixpkgsOverlays ++ [(import ./nixpkgs-overlay.nix { inherit nix-thunk; })]; } ) );

    # You must accept the Android Software Development Kit License Agreement at
    # https://developer.android.com/studio/terms in order to build Android apps.
    # Uncomment and set this to `true` to indicate your acceptance:
    config.android_sdk.accept_license = true;

    # In order to use Let's Encrypt for HTTPS deployments you must accept
    # their terms of service at https://letsencrypt.org/repository/.
    # Uncomment and set this to `true` to indicate your acceptance:
    # terms.security.acme.acceptTerms = false;
  }
}:
with obelisk;
project ./. ({ pkgs, ... }: {
  # Not using "storage.private" (our newer domain name) since that makes the build
  # fail: "[...] is not a valid Java package name as 'private' is a Java keyword."
  android.applicationId = "io.privatestorage.privatestoragemobile";
  android.displayName = "Private Storage Mobile";
  ios.bundleIdentifier = "systems.obsidian.obelisk.examples.minimal";
  ios.bundleName = "Obelisk Minimal Example";

  # Override some elements of the Haskell package set - get newer versions,
  # add custom libraries that aren't packaged upstream, etc.
  overrides = pkgs.callPackage ./haskell-overrides.nix {
    inherit nix-thunk;
  };
}) // {
  # Make sure we use the same nixpkgs here as we gave to obelisk /
  # reflex-platform above.
  inherit (obelisk) nixpkgs;
}
