-- | Implement platform-specific path-related operations for the Linux
-- platform.
module FrontendPaths (getFilesDir) where

import System.Environment.XDG.BaseDir (getUserDataDir)
import System.Directory (createDirectoryIfMissing)

-- | Get the path to a directory that is specific to the application where
-- internal application data may be written (and read back).
getFilesDir :: IO (Maybe FilePath)
getFilesDir = do
  -- It's not likely this is really the "mobile" case but we want to avoid
  -- colliding with PrivateStorageDesktop which uses "privatestorage".
  dataPath <- getUserDataDir "privatestoragemobile"

  createDirectoryIfMissing True dataPath
  pure $ Just dataPath
