{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE LambdaCase #-}
{-# LANGUAGE RecursiveDo #-}

module Controller (activePage) where

import Obelisk.Frontend
import Reflex.Dom.Core

import Pages.Folder (folderPage)
import Pages.TechDemo (techDemoPage)
import Pages.DownloadOneCap (downloadOneCapPage)

-- dyn :: (Adjustable t m, NotReady t m, PostBuild t m) => Dynamic t (m a) -> m (Event t a)
-- constDyn :: Reflex t => a -> Dynamic t a
-- holdDyn :: a -> Event t a -> m (Dynamic t a)
-- switch :: Behavior t (Event t a) -> Event t a
-- current :: Dynamic t a -> Behavior t a

-- widgetHold :: (Adjustable t m, MonadHold t m) => m a -> Event t (m a) -> m (Dynamic t a)
-- switchHold ??

-- el' :: forall t m a. DomBuilder t m => Text -> m a -> m (Element EventResult (DomBuilderSpace m) t, a)
-- button :: DomBuilder t m => Text -> m (Event t ())

-- | An enumeration of the views to which it is possible to switch.
data WhichPage = TechDemo | FolderPage | DownloadOneCap

-- | A view onto whichever page is currently active, as determined by user
-- inputs to the app.
--
-- This version just switches back and forth between the two views that exist
-- now - tech demo and folder contents.
activePage :: ObeliskWidget t r m => m ()
activePage =
  do
    rec
      -- Event t (m (Event t WhichPage))
      let activePageEv = pickPage <$> switchEv

      -- Dynamic (Event t WhichPage)
      widgetDyn <- widgetHold (pickPage DownloadOneCap) activePageEv

      -- Behavior t (Event t WhichPage)
      let widgetBhr = current widgetDyn

      -- Event t WhichPage
      let switchEv = switch widgetBhr

    pure ()

-- | Get the page which should be displayed for a given choice.
--
-- This also currently hard codes the back-and-forth logic by replacing each
-- page's event value with an appropriate value of WhichPage.
pickPage :: ObeliskWidget t r m => WhichPage -> m (Event t WhichPage)
pickPage TechDemo = fmap (FolderPage <$) techDemoPage
pickPage FolderPage = fmap (TechDemo <$) folderPage
pickPage DownloadOneCap = fmap (FolderPage <$) downloadOneCapPage
