{-# LANGUAGE NamedFieldPuns #-}
{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE TypeFamilies #-}

module Pages.DownloadOneCap (downloadOneCapPage) where

import Control.Monad.IO.Class (liftIO)
import Data.List.NonEmpty (NonEmpty(..))

import Obelisk.Frontend
import Reflex.Dom.Core

import Tahoe.CHK.Types (StorageServer)
import Tahoe.CHK.Capability (pCapability)
import Text.Megaparsec (parse)

import TahoeDownloader

data TahoeConfiguration =
    TahoeConfiguration
    { storageServers :: NonEmpty StorageServer
    } deriving (Eq, Ord, Show)

downloadOneCapPage :: ObeliskWidget t r m => m (Event t ())
downloadOneCapPage TahoeConfiguration { storageServers } = do
  -- Place an input field into the page and retain a reference to it.
  el "span" $ text "Download: "
  capInputWidget <- inputElement def

  -- `Dynamic t T.Text` holding the value of the text input.
  let capTextDyn = _inputElement_value capInputWidget

  -- `Dynamic t CHK` holding parsed capabilities.
  let parsedCap = filterRight $ parse pCapability "" <$> capTextDyn

  sharesEv <- performEvent $ ffor parsedCap $ liftIO . downloadShareBytes $ \case ->
              (CHKVerifier v) ->  v
              (CHKReader (Reader { verifier })) -> verifier

  plaintextEv <- decodeFull <$> sharesEv
  plaintextDyn <- holdDyn "" plaintextEv

  el "div" $ do
    el "div" $ text "Plaintext: "
    el "div" $ dynText plaintextDyn
