{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE OverloadedStrings #-}

module Pages.Folder (folderPage) where

import Reflex.Material.List
import Reflex.Material.Types

import Data.Tuple.Extra (uncurry3)
import Data.Text
import Obelisk.Frontend
import Reflex.Dom.Core

-- | Define the entire view for looking at a single folder and its direct
-- contents.
folderPage :: ObeliskWidget t r m => m (Event t ())
folderPage = do
      leaveEv <- elClass "header" "mdc-top-app-bar mdc-ripple-surface" $
        divClass "mdc-top-app-bar__row" $
          elClass "section" "mdc-top-app-bar__section mdc-top-app-bar__section--align-start" $ do
            (closeButton, _) <- elAttr' "button" ("class" =: "material-icons mdc-top-app-bar__navigation-icon mdc-icon-button" <> "aria-label" =: "Close") $
              text "close"
            elClass "span" "mdc-top-app-bar__title" $
              text "Documents"

            elAttr "section" ("class" =: "mdc-top-app-bar__section mdc-top-app-bar__section--align-end" <> "role" =: "toolbar") $ do
              elAttr "button" ("class" =: "material-icons mdc-top-app-bar__action-item mdc-icon-button" <> "aria-label" =: "Share") $
                text "share"
              elAttr "button" ("class" =: "material-icons mdc-top-app-bar__action-item mdc-icon-button" <> "aria-label" =: "Delete") $
                text "delete"
              elAttr "button" ("Class" =: "material-icons mdc-top-app-bar__action-item mdc-icon-button" <> "aria-label" =: "Open menu") $
                text "more_vert"

            pure (domEvent Click closeButton)

      elClass "main" "mdc-top-app-bar--fixed-adjust" $ do

        list_ "ul" (mdcListTwoLine_ <> CssClass "mdc-list" <> CssClass "inline-demo-list") $
                mapM_
                     (uncurry3 folderEntryWidget)
                     [ ("folder_open", "Taxes", "1 file; 4 MB")
                     , ("insert_drive_file", "Notes.txt", "3 MB")
                     , ("insert_drive_file", "2023 Resume.pdf", "5 MB")
                     ]

        el "script" $
          text "mdc.ripple.MDCRipple.attachTo(document.querySelector('.mdc-ripple-surface'));"

      pure leaveEv


-- | Define the view for a single item in a folder, including an icon, the
-- item name, and some metadata.
folderEntryWidget :: ObeliskWidget t r m => Text -> Text -> Text -> m ()
folderEntryWidget icon name info =
    elClass "li" "mdc-list-item mdc-ripple-surface" $ do
      elAttr "span" ("class" =: "mdc-list-item__graphic material-icons" <> "aria-hidden" =: "true") $
        text icon
      elAttr "span" ("class" =: "mdc-list-item__text") $ do
        elAttr "span" ("class" =: "mdc-list-item__primary-text") $
          text name
        elAttr "span" ("class" =: "mdc-list-item__secondary-text") $
          text info
