{-# LANGUAGE NamedFieldPuns #-}

module TahoeDownloader (downloadShares) where

import Tahoe.CHK.Decode (Share)
import Tahoe.CHK.Capability (Verifier(..))
import Tahoe.CHK.Types (StorageServer)
import qualified Data.ByteString as BS

type ShareNumber = Int

type ShareMap = Map ShareNumber [StorageServer]

-- | Identify which servers claim to have some data at some index.
locateShares ::
    -- | All of the servers which could possibly have the data.  These "could
    -- possibly" have the data because local configuration suggests the data
    -- might have been uploaded to them in the past.
    [StorageServer] ->
    -- | The storage index at which to look for data.
    StorageIndex ->
    -- | The share numbers to look for.
    Set ShareNumber ->
    -- | A mapping from share number to servers which claim to have the data.
    IO ()

-- | Download the bytes for one share number at some index from some server.
downloadShare ::
    -- | All of the servers which likely have a certain share number at the
    -- index.  These "likely" have the data because a recent ``locateShares``
    -- call found it there.
    [StorageServer] ->
    -- | The storage index at which to look for data.
    StorageIndex ->
    -- | The share number to look for.
    ShareNumber ->
    -- | Either an error encountered trying to get the data or the data itself.
    IO (Either DownloadError ByteString)

type Required = Int

-- | Find and download enough shares to decode the data identified by the given capability.
fetchShares ::
    [StorageServer] ->
    StorageIndex ->
    Required ->
    IO [(ShareNumber, ByteString)]

-- | Maybe something like the number of currently valid shares that can be lost
-- before the object is lost.
type Health = Int

-- from tahoe-chk
-- verifyFull :: Verifier -> [(ShareNumber, ByteString)] -> Either VerifyError Health
-- decodeFull :: Reader -> [(ShareNumber, ByteString)] -> Either VerifyError ByteString
